Coverage for src/abcd_graph/graph/core/utils.py: 100%

1# Copyright (c) 2024 Jordan Barrett & Aleksander Wojnarowicz

2#

3# Permission is hereby granted, free of charge, to any person obtaining a copy

4# of this software and associated documentation files (the "Software"), to deal

5# in the Software without restriction, including without limitation the rights

6# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell

7# copies of the Software, and to permit persons to whom the Software is

8# furnished to do so, subject to the following conditions:

9#

10# The above copyright notice and this permission notice shall be included in all

11# copies or substantial portions of the Software.

12#

13# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR

14# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,

15# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE

16# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER

17# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,

18# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE

19# SOFTWARE.

20

21 __all__ = ["rand_round", "powerlaw_distribution", "get_community_color_map"]

22

23 import math

24 import random

25 from typing import TYPE_CHECKING

26

27 import numpy as np

28 from numpy.typing import NDArray

29

30 if TYPE_CHECKING: # pragma: no cover

31 from abcd_graph.graph.core.abcd_objects import Community

32

33

34 def rand_round(x: float) -> int:

35 p = x - math.floor(x)

36 return int(math.floor(x) + 1) if random.uniform(0, 1) <= p else int(math.floor(x))

37

38

39 def powerlaw_distribution(choices: NDArray[np.int64], intensity: float) -> NDArray[np.float64]:

40 dist: NDArray[np.float64] = (choices ** (-intensity)) / np.sum(choices ** (-intensity))

41 return dist

42

43

44 def get_community_color_map(communities: list["Community"]) -> list[str]:

45 import matplotlib.colors as colors # type: ignore[import]

46

47 colors_list = list(colors.BASE_COLORS.values())[: len(communities)]

48

49 color_map = []

50

51 for i, community in enumerate(communities):

52 color = colors_list[i]

53 color_map.extend([color] * len(community.vertices))

54

55 return color_map